/*---------------------------------------------------------------------------------------------------------------------------------------------------
 * irmp-main-avr-uart.c - demo main module to test IRMP decoder on AVR with UART
 *
 * Copyright (c) 2009-2016 Frank Meyer - frank(at)fli4l.de
 *
 * $Id: irmp-main-avr-uart.c,v 1.4 2016/11/18 11:48:47 fm Exp $
 *
 * This demo module is runnable on AVRs with UART
 *
 * ATMEGA88 @ 8 MHz internal RC      Osc with BODLEVEL 4.3V: lfuse: 0xE2 hfuse: 0xDC efuse: 0xF9
 * ATMEGA88 @ 8 MHz external Crystal Osc with BODLEVEL 4.3V: lfuse: 0xFF hfuse: 0xDC efuse: 0xF9
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */

#include <string.h>
#include <util/atomic.h>
#include "irmp.h"
#include "fifo.h"
#include "ws2812.h"

#ifndef F_CPU
#error F_CPU unknown
#endif

static void uart_puts_P (PGM_P s);

#define SYNC_LOW   PORTB  &= ~(1<<PB0);
#define SYNC_HIGH  PORTB  |=  (1<<PB0);
#define SYNC_TOGGLE PORTB ^=  (1<<PB0);

#define LED_CNT 300         // number of LEDs in string
#define SNAKE_LEN 10        // length of LED snake
#define FIFO_SIZE 64        // size of IRMP FIFO /bytes

uint8_t fifo_data[FIFO_SIZE];
fifo_t irmp_fifo;
volatile uint8_t flag_20ms;
volatile uint8_t flag_sync_fast;
volatile uint8_t cnt_2kHz;
uint8_t led[LED_CNT][3];    // GRB data for LED string
IRMP_DATA irmp_data;
uint8_t test_cnt=1, test_cnt_max=10, test_sel=0, test_on=1;

uint8_t test1;
int test2;
int head, tail;
uint32_t color;

// simple toggle test

void toggle_test(void) {

    if (test1) {
        led[4][0] = 255;
        led[7][1] = 0;
        led[290][0] = 255;
        led[295][1] = 0;
    } else {
        led[4][0] = 0;
        led[7][1] = 255;
        led[290][0] = 0;
        led[295][1] = 255;
    }

    test1 = !test1;
}

// single step RGB test

void single_color(void) {

    test2++;
    if (test2==LED_CNT*3) test2=0;
    memset(led, 0, sizeof(led));
    led[test2 / 3][ test2 % 3] = 255;
}

// snake test with colors

void snake_test(void) {

    if (tail >= 0) {
        led[tail][0]=0;
        led[tail][1]=0;
        led[tail][2]=0;
    }
    if (head >= 0) {
        led[head][0]=color;
        led[head][1]=color>>8;
        led[head][2]=color>>16;
    }
    tail++;
    if (tail >= LED_CNT) {
        tail = -SNAKE_LEN;
    }
    head++;
    if (head >= LED_CNT) {
        head = -SNAKE_LEN;
    }

    if (color < 0x000100) {
        color += 0x000004;
    } else if (color < 0x010000) {
        color += 0x000400;
    } else if (color < 0x1000000) {
        color += 0x040000;
    } else {
        color = 0;
    }
}

void test_reset(void) {
    test1 = 0;
    test2 = 0;
    head  = 0;
    tail  = -SNAKE_LEN;
    color = 0; 
}

// LED update

void led_update(void) {
    if (flag_20ms == 1) { 
        flag_20ms++;             
        //SYNC_HIGH  
        if (fifo_get_free(&irmp_fifo) > 24) {
            flag_sync_fast=0;
            while(!flag_sync_fast);     // wait for 8 bit sampling interval
            ATOMIC_BLOCK(ATOMIC_RESTORESTATE) {
               // SYNC_HIGH
                ws2812_write_and_sample((uint8_t*)led, LED_CNT*3, &PORTB, (1<<PB2), &irmp_fifo);
                //ws2812_write((uint8_t*)led, LED_CNT*3, &PORTB, (1<<PB2));                
                //SYNC_LOW
            }             
        } else {
            uart_puts_P (PSTR("FIFO overflow expected!\r\n"));
            ATOMIC_BLOCK(ATOMIC_RESTORESTATE) {
                fifo_init(&irmp_fifo, fifo_data, FIFO_SIZE);        // reset FIFO
            }
        }
        //SYNC_LOW
    }
}

// process IR data offline

void ir_sample_processing(void) {
    uint8_t shift, mask;

    if (fifo_get_level(&irmp_fifo) > 0) {
        shift = fifo_read(&irmp_fifo);
        for (mask=0x80; mask != 0; mask >>= 1) {
            irmp_ISR(shift & mask);    
        }
    }
}    

// execute received commands

void ir_protocol_processing(void) {
    uint8_t cmd_available;
    
    cmd_available = irmp_get_data(&irmp_data);  
    // ignor repeated button presses  
    if (cmd_available && (irmp_data.flags == 0x00)) {
        switch (irmp_data.command) {
            case 0x10: // volume +
                if (test_cnt_max >1) {
                    test_cnt_max--;    // speed up
                    uart_puts_P (PSTR("Speed +\r\n"));
                } else {
                    uart_puts_P (PSTR("Max Speed\r\n"));                
                }
            break;

            case 0x11: // volume -
                if (test_cnt_max <100) {
                    test_cnt_max++;  // speed down
                    uart_puts_P (PSTR("Speed -\r\n"));
                } else {
                    uart_puts_P (PSTR("Min Speed\r\n"));                
                }
            break;

            case 0x20: // program +
                if (test_sel < 2) {
                    test_sel++;          // next test
                    uart_puts_P (PSTR("Test +\r\n"));
                    memset(led, 0, sizeof(led));
                    test_reset();
                } else {
                    uart_puts_P (PSTR("Max Test\r\n"));                
                }
            break;

            case 0x21: // program -
                if (test_sel > 0) {
                    test_sel--;          // previous test
                    uart_puts_P (PSTR("Test -\r\n"));
                    memset(led, 0, sizeof(led));
                    test_reset();
                } else {
                    uart_puts_P (PSTR("Min Test\r\n"));                
                }
            break;

            case 0x0C: // power on/off
                test_on = !test_on;
                if (!test_on) {
                    memset(led, 0, sizeof(led));
                    uart_puts_P (PSTR("Power off\r\n"));
                } else {
                    uart_puts_P (PSTR("Power on\r\n"));
                }
            break;

        }
    }
}

// LED test control

void led_test_control(void) {
    if (flag_20ms == 2) {
        flag_20ms = 0;
        if (test_on) {
            test_cnt--;
            if (test_cnt == 0) {
                test_cnt = test_cnt_max;
                switch (test_sel) {
                    case 0: toggle_test(); break;
                    case 1: single_color(); break;
                    case 2: snake_test(); break;
                }
            }
           
        } else {
            test_cnt=1;
        }
    }     
}

/*---------------------------------------------------------------------------------------------------------------------------------------------------
 * ATMEL AVR part:
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */
#define BAUD  9600L
#include <util/setbaud.h>

#ifdef UBRR0H

#define UART0_UBRRH                             UBRR0H
#define UART0_UBRRL                             UBRR0L
#define UART0_UCSRA                             UCSR0A
#define UART0_UCSRB                             UCSR0B
#define UART0_UCSRC                             UCSR0C
#define UART0_UDRE_BIT_VALUE                    (1<<UDRE0)
#define UART0_UCSZ1_BIT_VALUE                   (1<<UCSZ01)
#define UART0_UCSZ0_BIT_VALUE                   (1<<UCSZ00)
#ifdef URSEL0
#define UART0_URSEL_BIT_VALUE                   (1<<URSEL0)
#else
#define UART0_URSEL_BIT_VALUE                   (0)
#endif
#define UART0_TXEN_BIT_VALUE                    (1<<TXEN0)
#define UART0_UDR                               UDR0
#define UART0_U2X                               U2X0

#else

#define UART0_UBRRH                             UBRRH
#define UART0_UBRRL                             UBRRL
#define UART0_UCSRA                             UCSRA
#define UART0_UCSRB                             UCSRB
#define UART0_UCSRC                             UCSRC
#define UART0_UDRE_BIT_VALUE                    (1<<UDRE)
#define UART0_UCSZ1_BIT_VALUE                   (1<<UCSZ1)
#define UART0_UCSZ0_BIT_VALUE                   (1<<UCSZ0)
#ifdef URSEL
#define UART0_URSEL_BIT_VALUE                   (1<<URSEL)
#else
#define UART0_URSEL_BIT_VALUE                   (0)
#endif
#define UART0_TXEN_BIT_VALUE                    (1<<TXEN)
#define UART0_UDR                               UDR
#define UART0_U2X                               U2X

#endif //UBRR0H

static void
uart_init (void)
{
    UART0_UBRRH = UBRRH_VALUE;                                                                      // set baud rate
    UART0_UBRRL = UBRRL_VALUE;

#if USE_2X
    UART0_UCSRA |= (1<<UART0_U2X);
#else
    UART0_UCSRA &= ~(1<<UART0_U2X);
#endif

    UART0_UCSRC = UART0_UCSZ1_BIT_VALUE | UART0_UCSZ0_BIT_VALUE | UART0_URSEL_BIT_VALUE;
    UART0_UCSRB |= UART0_TXEN_BIT_VALUE;                                                            // enable UART TX
}

static void
uart_putc (unsigned char ch)
{
    while (!(UART0_UCSRA & UART0_UDRE_BIT_VALUE))
    {
        ;
    }

    UART0_UDR = ch;
}

static void
uart_puts (char * s)
{
    while (*s)
    {
        uart_putc (*s);
        s++;
    }
}

static void
uart_puts_P (PGM_P s)
{
    uint8_t ch;

    while ((ch = pgm_read_byte(s)) != '\0')
    {
        uart_putc (ch);
        s++;
    }
}

static char *
itoh (char * buf, uint8_t digits, uint16_t number)
{
    for (buf[digits] = 0; digits--; number >>= 4)
    {
        buf[digits] = "0123456789ABCDEF"[number & 0x0F];
    }
    return buf;
}

static void
timer1_init (void)
{
#if defined (__AVR_ATtiny45__) || defined (__AVR_ATtiny85__)                // ATtiny45 / ATtiny85:

#if F_CPU >= 16000000L
    OCR1C   =  (F_CPU / F_INTERRUPTS / 8) - 1;                              // compare value: 1/15000 of CPU frequency, presc = 8
    TCCR1   = (1 << CTC1) | (1 << CS12);                                    // switch CTC Mode on, set prescaler to 8
#else
    OCR1C   =  (F_CPU / F_INTERRUPTS / 4) - 1;                              // compare value: 1/15000 of CPU frequency, presc = 4
    TCCR1   = (1 << CTC1) | (1 << CS11) | (1 << CS10);                      // switch CTC Mode on, set prescaler to 4
#endif

#else                                                                       // ATmegaXX:
    OCR1A   =  (F_CPU / F_INTERRUPTS) - 1;                                  // compare value: 1/15000 of CPU frequency
    TCCR1B  = (1 << WGM12) | (1 << CS10);                                   // switch CTC Mode on, set prescaler to 1
#endif

#ifdef TIMSK1
    TIMSK1  = 1 << OCIE1A;                                                  // OCIE1A: Interrupt by timer compare
#else
    TIMSK   = 1 << OCIE1A;                                                  // OCIE1A: Interrupt by timer compare
#endif
}

#ifdef TIM1_COMPA_vect                                                      // ATtiny84
#define COMPA_VECT  TIM1_COMPA_vect
#else
#define COMPA_VECT  TIMER1_COMPA_vect                                       // ATmega
#endif

// Timer1 output compare A interrupt service routine, called with F_INTERRUPTS Hz
// only sampling of IR data and storage in FIFO

ISR(COMPA_VECT) {
    static uint8_t ir_data, mask = 0x80;

    SYNC_HIGH
    // sample IR data and store 8 bits locally before writing to FIFO
    if (PINL & (1<<PL1)) {
        ir_data |=  mask;
    }
    mask >>= 1;

    // phase shift between 20ms flag and fast sync
    // to minimize wasted synchronization time

    if (mask == 0x01) {
        if (++cnt_2kHz >= 40) {
            cnt_2kHz = 0;
            flag_20ms = 1;
            //SYNC_TOGGLE
            //SYNC_LOW
        }
    }

    if (mask == 0) {
        mask = 0x80;
        flag_sync_fast = 1;
        fifo_write_ISR(&irmp_fifo, ir_data);
        ir_data = 0;
    }
    SYNC_LOW
}

int main (void) {

    // set to output for LED-Data and SYNC
    DDRB = (1<<PB2) | (1<<PB0);     

    irmp_init();                                                            // initialize irmp
    timer1_init();                                                          // initialize timer1
    uart_init();                                                            // initialize uart
    fifo_init(&irmp_fifo, fifo_data, FIFO_SIZE);
    test_reset();
    sei ();                                                                 // enable interrupts

    uart_puts_P (PSTR("IRMP + WS2812B\r\n"));

    while (1) {
        led_update();
        ir_sample_processing();
        ir_protocol_processing();
        led_test_control();
    }
}
